/**************************************************************************/
/* svgafft --  Spectrum Analyzer                                          */
/*                                                                        */ 
/* bar configuration header file                                          */
/**************************************************************************/

/**************************************************************************
 
    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: barconfig.h,v 1.2 1995/01/12 00:08:13 root Exp $             

#ifndef _BARCONFIGURATION_INCLUDED
#define _BARCONFIGURATION_INCLUDED

// define this if you want to be able
// to change display types at runtime

/*******************************************************************/
#define RUNTIME_BARCHANGE
/*******************************************************************/


/*******************************************************************/
#ifdef RUNTIME_BARCHANGE

// keep the following two lines unchanged
class BarGraph;
typedef BarGraph BaseType;

/*******************************************************************/
#else
/*******************************************************************/

// uncomment one of the following bar display types 
// if RUNTIME_BARCHANGE is disabled
class PeakDropSegmentedBarGraph;
typedef PeakDropSegmentedBarGraph BaseType;

#endif
/*******************************************************************/

/*******************************************************************/
#define FWIDTH 8
#define FHEIGHT 8
#define FCOLOR 178
/*******************************************************************/

// defines for the fading segment class
#define MAXNUMFADECOLS 32

#ifdef 1
#define IFADE_R 128
#define IFADE_G 255
#define IFADE_B 128
#else
#define IFADE_R 128
#define IFADE_G 255
#define IFADE_B 0
#endif

#define FFADE_R 0
#define FFADE_G 0
#define FFADE_B 0
// end defines for fading segment class
/*******************************************************************/


/*******************************************************************/
#endif _BARCONFIGURATION_INCLUDED
/*******************************************************************/
