/**************************************************************************/
/* svgafft --  Spectrum Analyzer                                          */
/*                                                                        */ 
/* screen defs                                                            */
/**************************************************************************/

/**************************************************************************

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: screen.h,v 1.3 1995/01/16 17:00:50 root Exp $             

#ifndef __SCREEN_INC
#define __SCREEN_INC

#define NUMSCREENS 5

struct aScreen { 
  int modenum;
  int rgbcols;
  int w;
  int h;
  int x1;
  int y1;
  int x2;
  int y2;
};

#endif __SCREEN_INC
