/**************************************************************************/
/* svgafft --  Spectrum Analyzer                                          */
/*                                                                        */ 
/* svga line display class interface definitions                          */
/**************************************************************************/

/**************************************************************************

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: s_line_display.h,v 0.1 1995/04/15 18:38:20 drewvel Exp $

/*******************************************************************/
#ifndef __S_LINE_DISPLAYH_INC
#define __S_LINE_DISPLAYH_INC

#include "svga_display.h"

/*******************************************************************/
// SVGA Line Display Class
/*******************************************************************/

/*******************************************************************/
class SVGA_Line_Display:public SVGA_Display {
protected:
  int *savedata;
  float intv;

public:
  SVGA_Line_Display(int n, Signal& S, 
		    aScreen &screen, int t=1,
		    boolean st=DEFLOGSCALE);
  ~SVGA_Line_Display();

  void RefreshFreqs();
  void Reset();
  void Show();
  void DrawInit();
  void ChangeScreen(int num);
  
  void ProcessControl(char control);

  void UpdateCoefficients();
};

#endif __S_LINE_DISPLAYH_INC
