/**************************************************************************/
/* svgafft --  Spectrum Analyzer                                          */
/*                                                                        */ 
/* screen defaults array                                                  */
/**************************************************************************/

/**************************************************************************

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: screen.cc,v 1.3 1995/01/16 17:00:49 root Exp $             

#include <vga.h>
#include "screen.h"

/*
   {
   modenumber, rgbcols, // modenum, rgbcols (0 || 1)
   width, height,
   x1, y1, // recangular size of default graph
   x2, y2
   }
   
*/

// make sure NUMSCREENS reflects the size
// of this array
aScreen Screens[NUMSCREENS]={
{
  G320x200x256, 1,
  320, 200,
  35, 30,
  317, 180
},
{
  G640x480x256, 1,
  640, 480,
  35, 30,
  630, 460
},
{
  G800x600x256, 1,
  800, 600,
  35, 30,
  780, 570
},
{
  G1024x768x256, 1,
  1024, 768,
  35, 30,
  1010, 746
},
{
  G640x480x16M, 0,
  640, 480,
  15, 30,
  630, 460
}
};
