/**************************************************************************/
/* svgafft --  Spectrum Analyzer                                          */
/*                                                                        */ 
/* rms stuff                                                              */
/**************************************************************************/

/**************************************************************************

    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: rms.h,v 1.1 1995/05/02 02:56:25 drewvel Exp $

 /*******************************************************************/
#include <math.h>

/*******************************************************************/
inline int irms(real buf[], int num)
{
  real avg=0;
  register int i;
  for (i=0; i<num; i++) avg+=buf[i]*buf[i];
  avg=sqrt(avg/num); // make sure num isn't 0... rather not check here
  return (int) avg; 
}


/*******************************************************************/
inline real rrms(real buf[], int num)
{
  real avg=0;
  register int i;
  for (i=0; i<num; i++) avg+=buf[i]*buf[i];
  avg=sqrt(avg/num);
  return avg; 
}


/*******************************************************************/
