/**************************************************************************/
/* svgafft --  Spectrum Analyzer                                          */
/*                                                                        */ 
/* boolean typedef header                                                 */
/**************************************************************************/

/**************************************************************************
 
    Copyright (C) 1995 Andrew Veliath

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


***************************************************************************/

// $Id: bools.h,v 0.2 1995/05/01 22:04:49 drewvel Exp $             

#ifndef _BOOLS_INCLUDED
#define _BOOLS_INCLUDED

#ifndef NEEDBOOLS

typedef bool boolean;

#else // need to make boolean type

typedef char boolean;

const int false=0;
const int true=1;

#endif 

#endif _BOOLS_INCLUDED
